/*
    Prepare the testing boilplate:
        - declare logging and counter utilities
        - clean table/database form previous run
        - set trigger fot logging the events
        - declare working buffers
*/


DEFINE new shared VARIABLE cnt AS INTEGER INIT 0 NO-UNDO. /* call counter */
FUNCTION x RETURNS INTEGER () :
   cnt = cnt + 1.
   RETURN cnt.
END FUNCTION.

FUNCTION d RETURNS LOGICAL (INPUT txt AS CHAR) :
   DISPLAY x() FORMAT ">>9" 
           txt FORMAT "x(50)"
       WITH FRAME df NO-LABELS NO-BOX. 
   RETURN YES.
END FUNCTION.

ON find of book            d("* FIND " + book.isbn).
ON create of book          d("* CREATE " + book.isbn).
ON ASSIGN of book.isbn     d("* ASSIGN isbn " + book.isbn).
ON ASSIGN of book.book-id  d("* ASSIGN id " + string(book.book-id)).
ON DELETE OF book          d("* DELETE " + book.isbn).
ON WRITE  OF book          d("* WRITE  " + book.isbn).

DO TRANSACTION:
   d("cleanup ...").
/*   DISABLE TRIGGERS FOR LOAD OF book.
   DISABLE TRIGGERS FOR DUMP OF book.*/

   FIND FIRST book WHERE book-id EQ 104 NO-ERROR.
   IF AVAILABLE book THEN DO:
      DELETE book.
   END.

   FIND FIRST book WHERE book-id EQ 101.
   book.isbn = "101".
   book.publisher = "".
   /* VALIDATE book. */

   FIND FIRST book WHERE book-id EQ 102.
   book.isbn = "102".
   book.publisher = "".
   /* VALIDATE book. */
   
   RELEASE book. 
   d("cleanup done").
END.

DEFINE new shared buffer boo_1 for book.
DEFINE new shared buffer boo_2 for book.

run write_delay/test2222-external1.t.
run write_delay/test2222-external2.t.

/*
  1 cleanup ...
  2 * FIND 101
  3 * FIND 102
      ? * ASSIGN isbn 102
      ? * WRITE  102  
  4 cleanup done
  5 about to start external block 1
  6   looking for book *102 in book_1
  7 * FIND 102
  8   about to start transaction block 1.1
  9     batch assigning new values to book_1
 10 * ASSIGN isbn c102
 11     assiged
 12     looking for book *102 in book_2
 13 * FIND c102
 14     validating book_2
 15 * WRITE  c102
 16     validated
 17   transaction block 1.1 ended
 18   about to start block 1.2
 19   looking for book *101 in book_1
 20 * FIND 101
 21     batch assigning new values to book_1
 22 * ASSIGN isbn a101
 23     assiged
 24     looking for book *101 in book_2
 25 * FIND a101
 26     validating book_2
 27 * WRITE  a101
 28     validated
 29     undoing and leaving block 1.2
 30 about to start external block 2
 31   looking for book *102 in book_2
 32 * FIND 102
 33   about to start transaction block 2.1
 34     batch assigning new values to book_1
 35 * ASSIGN isbn c102
 36     assiged
 37     looking for book *102 in book_2
 38 * FIND c102
 39     validating book_2
 40 * WRITE  c102
 41     validated
 42   transaction block 2.1 ended
 43   about to start block 2.2
 44   looking for book *101 in book_1
 45 * FIND 101
 46     batch assigning new values to book_1
 47 * ASSIGN isbn a101
 48     assiged
 
 51     looking for book *101 in book_2
 52 * FIND a101
 53     validating book_2
 54 * WRITE  a101
 55     validated
 56     undoing and leaving block 2.2
 57   block 2.1 ended
 58 external block 2 ended
*/
