DEFINE VARIABLE cnt AS INTEGER INIT 0 NO-UNDO.

/* this function outputs (and counts) the lines to a single frame so they can be easily view
   in the order they were generated */
FUNCTION d RETURNS LOGICAL(INPUT s AS CHARACTER, INPUT p1 AS CHARACTER, INPUT p2 AS CHARACTER):
   cnt = cnt + 1.
   DISPLAY
      cnt FORMAT ">9"
      s   FORMAT "x(20)"
      p1
      p2
      SKIP WITH FRAME f0 WITH NO-BOX NO-LABELS.
END FUNCTION.

/* initial cleanup */
FIND book WHERE book.book-id = 69896 NO-ERROR.
IF AVAILABLE(book) THEN
DO:
   d("found isbn-book", book.isbn, "").      /* line #1 */
   DELETE book.
END.
ELSE
   cnt = cnt + 1.                            /* keep then counter in sync */

ON WRITE OF book OLD BUFFER obook
    d("Writing book", book.isbn, obook.isbn).

/* creating test record */
CREATE book.
ASSIGN book.isbn = "varza"
       book.book-id = 69896.
DISPLAY "New book assigned" SKIP WITH FRAME f0 WITH NO-BOX NO-LABELS.

FIND FIRST book WHERE book.book-id EQ 69896. /* will trigger flush/WRITE event #2 */
IF cnt NE 2 THEN DO:
    MESSAGE "Write trigger failed to execute!".
   cnt = cnt + 1.                            /* keep then counter in sync */
END.

ON ASSIGN OF book.isbn OLD VALUE ov DO:
    isbn = "'" + isbn + "'".
    d("Assigning book", isbn, ov).
END.

d("Before validate", "", "").                /* line #3 */
VALIDATE book.                               /* should not trigger WRITE, because record was freshened from db */

book.isbn = "mere".                          /* will trigger ASSIGN #4 */
d("Record altered to", book.isbn, "mere").   /* line #5 */

VALIDATE book.                               /* should trigger WRITE #6 */
IF cnt NE 6 THEN DO:
    MESSAGE "Write trigger failed to execute!".
    cnt = cnt + 1.                           /* keep then counter in sync */
END.

DO TRANSACTION:
    book.isbn = "pere".                      /* will trigger ASSIGN #7 */
    UNDO, LEAVE.                             /* undo/leaving won't trigger ASSIGN */
END.

VALIDATE book.                               /* should not trigger WRITE event, record actually wasn't changed because then undo rollback */
IF cnt NE 7 THEN DO:
    MESSAGE "Extra WRITE trigger executed!".
    cnt = cnt - 1.                           /* keep then counter in sync */
END.

DO TRANSACTION:
    book.isbn = "prune".                     /* will trigger ASSIGN #8 */
    VALIDATE book.                           /* should trigger WRITE #9 */

    IF cnt NE 9 THEN DO:
        MESSAGE "WRITE trigger failed to execute!".
        cnt = cnt + 1.                       /* keep then counter in sync */
    END.

    UNDO, LEAVE.                             /* strange thing happens here. book.isbn is undone to 'mere',
                                                along with persisted db record, BUT no trigger disable fired. The record IS valid. */
END.

d("Yet, isbn is", book.isbn, "").            /* line #10 */
VALIDATE book.                               /* no effect */
d("After validation", book.isbn, "").        /* line #11 */

FIND book WHERE book.book-id = 69896.        /* no visible effect */
d("After refresh", book.isbn, "").           /* line #12 */

book.isbn = "mere".                          /* will fire ASSIGN trigger #13 */
VALIDATE book.                               /* will fire the WRITE trigger #14 even if the record actually did not change */

VALIDATE book.                               /* no-op */
book.isbn = "'mere'".                        /* no-op, Assign not fired */

d("about to end", "", "").                   /* line #15 */
book.isbn = "mere".                          /* will fire ASSIGN trigger #16 */

FUNCTION quitApp RETURNS LOGICAL ():
   QUIT.
END FUNCTION.

/* quitApp().                                   -- will fire the WRITE trigger #17 anyway at the end of the app */

/* end of procedure will fire WRITE trigger #17 */


/* Expected output:

 1 found isbn-book      'mere'
New book assigned
 2 Writing book         varza
 3 Before validate
 4 Assigning book       'mere'   varza
 5 Record altered to    'mere'   mere
 6 Writing book         'mere'   varza
 7 Assigning book       'pere'   'mere'
 8 Assigning book       'prune'  'mere'
 9 Writing book         'prune'  'mere'
10 Yet, isbn is         'mere'
11 After validation     'mere'
12 After refresh        'mere'
13 Assigning book       'mere'   'mere'
14 Writing book         'mere'   'mere'
15 about to end
16 Assigning book       'mere'   'mere'
17 Writing book         'mere'   'mere'

*/
